package cz.muni.fi.pv168.swing;

import cz.muni.fi.pv168.books.Book;
import cz.muni.fi.pv168.books.BookException;
import cz.muni.fi.pv168.books.BookManager;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import javax.swing.table.DefaultTableModel;
import java.util.List;
import java.util.ResourceBundle;

/**
 * Created by makub on 5/7/14.
 */
public class BooksTableModel extends DefaultTableModel {

    private final BookManager bookManager;
    List<Book> books;

    final static Logger log = LoggerFactory.getLogger(BooksTableModel.class);

    public BooksTableModel(BookManager bookManager) {
        this.bookManager = bookManager;
        try {
            books = bookManager.getAllBooks();
            log.debug("bookManager={}", bookManager);
        } catch (BookException e) {
            log.error("cannot read books", e);
        }
    }

    @Override
    public int getRowCount() {
        return books != null ? books.size() : 0;
    }

    @Override
    public int getColumnCount() {
        return 2;
    }

    static ResourceBundle texts = ResourceBundle.getBundle("Texts");

    @Override
    public String getColumnName(int column) {
        switch (column) {
            case 0:
                return texts.getString("author");
            case 1:
                return texts.getString("name");
            default:
                throw new IllegalArgumentException("no columnnumber " + column);
        }
    }

    @Override
    public Object getValueAt(int row, int column) {
        Book book = books.get(row);
        switch (column) {
            case 0:
                return book.getAuthor();
            case 1:
                return book.getName();
            default:
                throw new IllegalArgumentException("no columnnumber " + column);
        }
    }

    public Book getBookForRow(int row) {
        return books.get(row);
    }

    public void deleteBookForRow(int row) {
        Book book = books.get(row);

        new SwingWorker<Void, Void>() {
            @Override
            protected Void doInBackground() throws Exception {
                try {
                    bookManager.deleteBook(book.getId());
                    books = bookManager.getAllBooks();
                } catch (BookException e) {
                    log.error("cannot delete book " + book, e);
                }
                return null;
            }

            @Override
            protected void done() {
                fireTableDataChanged();
            }
        }.execute();



    }
}
